package vm

import (
	"testing"
	"time"

	"github.com/gnolang/gno/tm2/pkg/crypto"
	"github.com/gnolang/gno/tm2/pkg/std"
	"github.com/gnolang/gno/tm2/pkg/store"
)

func TestVMCrash(t *testing.T) {
	addr := crypto.AddressFromPreimage([]byte("addr1"))
	coins := std.MustParseCoins("")

	// Create the fuzzed file content
	fileContent := `package main
func main() {
    buffer := make([]int, 1_000_000_000_000)
	buffer[1] = 1
}
`

	files := []*std.MemFile{
		{"script.gno", fileContent},
	}

	msg := NewMsgRun(addr, coins, files)

	// Run the VM
	env := setupTestEnv()
	ctx := env.ctx.WithGasMeter(store.NewGasMeter(1_000_000))
	acc := env.acck.NewAccountWithAddress(ctx, addr)
	env.acck.SetAccount(ctx, acc)

	start := time.Now()

	func() {
		defer func() {
			if r := recover(); r != nil {
				switch r.(type) {
				case store.OutOfGasException: // panic in consumeGas()
					break
				default:
					panic(r)
				}
			}
		}()
		_, err := env.vmk.Run(ctx, msg)
		if err != nil {
			t.Logf("Error occurred: %.100s", err)
		}
	}()

	duration := time.Since(start)
	if duration > 1000*time.Millisecond {
		t.Logf("Execution took %v", duration)
	}
}
